@@ -1,6 +1,6 @@
 /*
  * yatm - Yet Another Time Machine
- * Copyright (C) 2004, 2005, 2006 Mario Lang
+ * Copyright (C) 2004, 2005, 2006, 2013 Mario Lang
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,11 +36,11 @@
 #include <speex/speex_callbacks.h>
 #include <speex/speex_header.h>
 #include <speex/speex_stereo.h>
-#include <vorbis/codec.h>
-#include <vorbis/vorbisfile.h>
 #include <soundtouch/SoundTouch.h>
 #include <ao/ao.h>
 
+#include <iostream>
+
 #include "config.h"
 
 static unsigned char verbosity = 1;
@@ -167,9 +167,8 @@ play_ao (int channels, int bufsize)
 }
 
 static void print_version();
-static int play_vorbis(int fd, const char *begin);
 static int play_speex(int fd, char *begin);
-static int play_sndfile (int fd, char *begin);
+static int play_sndfile (int fd, char const *begin, char const *end);
 static int play_mpeg(int fd, char *begin, char *end);
 
 int
@@ -213,72 +212,69 @@ main (int argc, char *argv[])
     input_file = strdup(argv[optind++]);
   }
   if (optind < argc) {
-    fprintf(stderr, "Excessive command line paramters, aborting...\n");
-    return 1;
+    std::cout << "Excessive command line paramters, aborting..." << std::endl;
+    exit(EXIT_FAILURE);
   }
   if (!input_file) {
-    fprintf(stderr, "No input file specified, aborting...\n");
+    std::cout << "No input file specified, aborting..." << std::endl;
+    exit(EXIT_FAILURE);
+  }
+
+  st = new SoundTouch();
+
+  struct sigaction action;
+
+  int fd = open(input_file, O_RDONLY);
+  if (fd == -1) {
+    fprintf(stderr, "Can not open %s: %s, aborting...\n", input_file, strerror(errno));
+    exit(EXIT_FAILURE);
+  }
+
+  ao_initialize();
+  audio_driver = ao_default_driver_id();
+
+  if (sigaction(SIGTSTP, 0, &save_sigtstp) == -1) {
+    fprintf(stderr, "Error saving sigtstp handler.\n");
+    exit(EXIT_FAILURE);
+  }
+  if (sigaction(SIGINT, 0, &save_sigint) == -1) {
+    fprintf(stderr, "Error saving sigint handler.\n");
     return 1;
-  } else {
-    int fd;
-
-    st = new SoundTouch();
-
-    struct sigaction action;
-
-    fd = open(input_file, O_RDONLY);
-    if (fd == -1) {
-      fprintf(stderr, "Can not open %s: %s, aborting...\n", input_file, strerror(errno));
-      return 1;
-    }
-
-    ao_initialize();
-    audio_driver = ao_default_driver_id();
-
-    if (sigaction(SIGTSTP, 0, &save_sigtstp) == -1) {
-      fprintf(stderr, "Error saving sigtstp handler.\n");
-      return 1;
-    }
-    if (sigaction(SIGINT, 0, &save_sigint) == -1) {
-      fprintf(stderr, "Error saving sigint handler.\n");
-      return 1;
-    }
-    action = save_sigtstp;
-    action.sa_handler = signal_handler;
-    sigemptyset(&action.sa_mask);
-    sigaddset(&action.sa_mask, SIGINT);
-    action.sa_flags = 0;
-    if (sigaction(SIGTSTP, &action, 0) == -1) {
-      fprintf(stderr, "Error setting sigtstp handler.\n");
-      return 0;
-    }
-    action = save_sigint;
-    action.sa_handler = signal_handler;
-    sigemptyset(&action.sa_mask);
-    sigaddset(&action.sa_mask, SIGTSTP);
-    action.sa_flags = 0;
-    if (sigaction(SIGTSTP, &action, 0) == -1) {
-      fprintf(stderr, "Error setting sigtstp handler.\n");
-      return 0;
-    }
-
-    initTTY();
-    st->setSetting(SETTING_USE_QUICKSEEK, 0);
-    st->setSetting(SETTING_USE_AA_FILTER, 1);
-    st->setPitch(powf(2.,pitchCentDelta/1200.));
-    st->setTempo(tempo);
-    if (!play_sndfile(fd, begin_time))
-      if (!play_vorbis(fd, begin_time))
-	if (!play_speex(fd, begin_time))
-	  play_mpeg(fd, begin_time, end_time);
-
-    if (begin_time) free(begin_time);
-    if (end_time) free(end_time);
-    close(fd);
-    delete st;
-    SLang_reset_tty();
+  }
+  action = save_sigtstp;
+  action.sa_handler = signal_handler;
+  sigemptyset(&action.sa_mask);
+  sigaddset(&action.sa_mask, SIGINT);
+  action.sa_flags = 0;
+  if (sigaction(SIGTSTP, &action, 0) == -1) {
+    fprintf(stderr, "Error setting sigtstp handler.\n");
     return 0;
   }
+  action = save_sigint;
+  action.sa_handler = signal_handler;
+  sigemptyset(&action.sa_mask);
+  sigaddset(&action.sa_mask, SIGTSTP);
+  action.sa_flags = 0;
+  if (sigaction(SIGTSTP, &action, 0) == -1) {
+    fprintf(stderr, "Error setting sigtstp handler.\n");
+    return 0;
+  }
+
+  initTTY();
+  st->setSetting(SETTING_USE_QUICKSEEK, 0);
+  st->setSetting(SETTING_USE_AA_FILTER, 1);
+  st->setPitch(powf(2.,pitchCentDelta/1200.));
+  st->setTempo(tempo);
+  if (!play_sndfile(fd, begin_time, end_time))
+    if (!play_speex(fd, begin_time))
+      play_mpeg(fd, begin_time, end_time);
+
+  if (begin_time) free(begin_time);
+  if (end_time) free(end_time);
+  close(fd);
+  delete st;
+  SLang_reset_tty();
+  return EXIT_SUCCESS;
 }
 
 /* MPEG */
@@ -595,7 +591,7 @@ parse_double_time (double *timer, char const *str)
   unsigned long seconds, fraction, fracpart;
   int minus;
 
-  while (isspace((unsigned char) *str)) ++str;
+  while (isspace(*str)) ++str;
   do {
     seconds = fraction = fracpart = 0;
 
@@ -639,7 +635,7 @@ parse_double_time (double *timer, char const *str)
     accum += time;
   } while (*str == '-' || *str == '+');
 
-  while (isspace((unsigned char) *str)) ++str;
+  while (isspace(*str)) ++str;
 
   if (*str != 0)
     return -1;
@@ -652,108 +648,7 @@ parse_double_time (double *timer, char const *str)
 static void
 print_version()
 {
-  fprintf(stderr, "YATM " VERSION "\n");
-}
-
-static int
-play_vorbis(int fd, const char *begin)
-{
-  FILE *stream;
-  OggVorbis_File vf;
-  vorbis_info *vi;
-  int current_section; /* An Ogg bitstream can have multiple sections */
-  int inSamples;
-  int eof = 0;
-  
-  stream = fdopen(dup(fd), "rb");
-  if(ov_open(stream, &vf, NULL, 0) < 0) {
-    fprintf(stderr,"Input does not appear to be an Ogg bitstream.\n");
-    rewind(stream);
-    fclose(stream);
-    return 0;
-  }
-  if ((vi = ov_info(&vf,-1)) == NULL) {
-    fprintf(stderr, "Unable to get Vorbis info\n");
-    ov_clear(&vf);
-    return 0;
-  }
-  fprintf(stderr, "Vorbis stream has %d channels and %dhz sampling rate.\n",
-	  vi->channels, vi->rate);
-  if (begin) {
-    double time;
-    int ret;
-    if (!ov_seekable(&vf)) {
-      fprintf(stderr, "Stream not seekable, sorry\n");
-    } else {
-      if (parse_double_time(&time, begin) == -1) {
-	fprintf(stderr, "Unable to parse time spec: %s\n", begin);
-	ov_clear(&vf);
-	fclose(stream);
-	return 1;
-      }
-      ret = ov_time_seek(&vf, time);
-      if (ret < 0) {
-	fprintf(stderr, "Unable to seek to position %f (error %d)\n",
-		time, ret);
-      }
-    }
-  }
-  audio_format.bits = 16;
-  audio_format.channels = vi->channels;
-  audio_format.rate = vi->rate;
-  audio_format.byte_format = AO_FMT_LITTLE;
-  if (audio_device) {
-    fprintf(stderr, "Audio device already open.\n");
-    ov_clear(&vf);
-    fclose(stream);
-    return 0;
-  }
-  audio_device = ao_open_live(audio_driver, &audio_format, NULL);
-  if (!audio_device) {
-    fprintf(stderr, "Error opening audio device: %d.\n", errno);
-    ov_clear(&vf);
-    fclose(stream);
-    return 0;
-  }
-  st->setSampleRate(vi->rate);
-  st->setChannels(vi->channels);
-  while (!eof) {
-    float **pcm;
-
-    pollKeyboard(NULL);
-    if (quit) goto close;
-    inSamples = ov_read_float(&vf, &pcm, 4096, &current_section);
-    if (inSamples == 0) {
-      eof = 1;
-    } else if (inSamples < 0) {
-      fprintf(stderr, "ov_read_float: %d\n", inSamples);
-    } else {
-      SAMPLETYPE samples[vi->channels * inSamples];
-      SAMPLETYPE *ptr = samples;
-      float *ch1 = pcm[0], *ch2 = pcm[1];
-
-      for (int i = 0; i < inSamples; i++) {
-        soundtouch::SAMPLETYPE val;
-	val = *ch1++*32768.f;
-	if (val > 32767) val = 32767;
-	else if (val < -32768) val = -32768;
-	*ptr++ = val;
-	if (vi->channels == 2) {
-	  val = *ch2++*32768.f;
-	  if (val > 32767) val = 32767;
-	  else if (val < -32768) val = -32768;
-	  *ptr++ = val;
-	}
-      }
-      st->putSamples(samples, inSamples);
-      play_ao(vi->channels, inSamples);
-    }
-  }
- close:
-  ao_close(audio_device);
-  ov_clear(&vf);
-  fclose(stream);
-  return 1;
+  fprintf(stderr, "YATM " YATM_VERSION "\n");
 }
 
 static int
@@ -964,8 +859,9 @@ seek_sndfile (float delta)
 }
 
 static int
-play_sndfile (int fd, char *begin)
+play_sndfile (int fd, char const *begin, char const *end)
 {
+  sf_count_t maxFrames = 0;
   memset (&sfinfo, 0, sizeof (sfinfo));
   if ((sndfile = sf_open_fd (dup(fd), SFM_READ, &sfinfo, TRUE))) {
     if (begin) {
@@ -976,6 +872,14 @@ play_sndfile (int fd, char *begin)
       }
       sf_seek(sndfile, (sf_count_t)(time * sfinfo.samplerate), SEEK_SET);
     }
+    if (end) {
+      double time;
+      if (parse_double_time(&time, end) == -1) {
+        fprintf(stderr, "Unable to parse end time spec: %s\n", end);
+        goto close;
+      }
+      maxFrames = (sf_count_t)(time * sfinfo.samplerate);
+    }
     audio_format.bits = 16;
     audio_format.channels = sfinfo.channels;
     audio_format.rate = sfinfo.samplerate;
@@ -996,13 +900,16 @@ play_sndfile (int fd, char *begin)
     {
       float buf[512 * sfinfo.channels];
       sf_count_t nFrames;
-      while ((nFrames = sf_readf_float(sndfile, buf, 512)) > 0) {
+      sf_count_t readFrames = 0;
+      while ((nFrames = sf_readf_float(sndfile, buf, 512)) > 0 && readFrames < maxFrames) {
+	if (readFrames + nFrames > maxFrames) nFrames = maxFrames - readFrames;
 	SAMPLETYPE samples[nFrames * sfinfo.channels];
 	int i;
 	for (i=0; i<nFrames*sfinfo.channels; i++) {
 	  samples[i] = buf[i]*32700.0;
 	}
 	st->putSamples(samples, nFrames);
+        readFrames += nFrames;
 	play_ao(sfinfo.channels, nFrames);
 	pollKeyboard(seek_sndfile);
 	if (quit) goto close;
